@@ -42,6 +42,7 @@ namespace X265_NS {
     {
     public:
         uint8_t           m_numEncodes;
+        uint8_t           m_numInputViews; // Number of inputs for multiview-extension
         PassEncoder        **m_passEnc;
         uint32_t           m_queueSize;
         ThreadSafeInteger  m_numActiveEncodes;
@@ -86,7 +87,7 @@ namespace X265_NS {
         x265_picture **m_outputRecon;
 
         CLIOptions m_cliopt;
-        InputFile* m_input;
+        InputFile* m_input[MAX_VIEWS];
         const char* m_reconPlayCmd;
         FILE*    m_qpfile;
         FILE*    m_zoneFile;
@@ -102,7 +103,7 @@ namespace X265_NS {
         void startThreads();
         void copyInfo(x265_analysis_data *src);
 
-        bool readPicture(x265_picture*);
+        bool readPicture(x265_picture*, int view);
         void destroy();
 
     private:
@@ -142,7 +143,7 @@ namespace X265_NS {
     public:
         PassEncoder *m_parentEnc;
         int m_id;
-        InputFile* m_input;
+        InputFile* m_input[MAX_VIEWS];
         int m_threadActive;
 
         Reader(int id, PassEncoder *parentEnc);
