@@ -20,7 +20,6 @@
  * This program is also available under a commercial proprietary license.
  * For more information, contact us at license @ x265.com.
  *****************************************************************************/
-
 #include "common.h"
 #include "bitstream.h"
 #include "param.h"
@@ -185,7 +184,7 @@ x265_encoder *x265_encoder_open(x265_param *p)
     // will detect and set profile/tier/level in VPS
     determineLevel(*param, encoder->m_vps);
 
-    if (!param->bAllowNonConformance && encoder->m_vps.ptl.profileIdc == Profile::NONE)
+    if (!param->bAllowNonConformance && encoder->m_vps.ptl.profileIdc[0] == Profile::NONE)
     {
         x265_log(param, X265_LOG_INFO, "non-conformant bitstreams not allowed (--allow-non-conformance)\n");
         goto fail;
@@ -357,11 +356,11 @@ int x265_encoder_reconfig(x265_encoder* enc, x265_param* param_in)
             VPS saveVPS;
             memcpy(&saveVPS.ptl, &encoder->m_vps.ptl, sizeof(saveVPS.ptl));
             determineLevel(*encoder->m_latestParam, encoder->m_vps);
-            if (saveVPS.ptl.profileIdc != encoder->m_vps.ptl.profileIdc || saveVPS.ptl.levelIdc != encoder->m_vps.ptl.levelIdc
+            if (saveVPS.ptl.profileIdc[0] != encoder->m_vps.ptl.profileIdc[0] || saveVPS.ptl.levelIdc != encoder->m_vps.ptl.levelIdc
                 || saveVPS.ptl.tierFlag != encoder->m_vps.ptl.tierFlag)
             {
                 x265_log(encoder->m_param, X265_LOG_WARNING, "Profile/Level/Tier has changed from %d/%d/%s to %d/%d/%s.Cannot reconfigure rate-control.\n",
-                         saveVPS.ptl.profileIdc, saveVPS.ptl.levelIdc, saveVPS.ptl.tierFlag ? "High" : "Main", encoder->m_vps.ptl.profileIdc,
+                         saveVPS.ptl.profileIdc[0], saveVPS.ptl.levelIdc, saveVPS.ptl.tierFlag ? "High" : "Main", encoder->m_vps.ptl.profileIdc[0],
                          encoder->m_vps.ptl.levelIdc, encoder->m_vps.ptl.tierFlag ? "High" : "Main");
                 x265_copy_params(encoder->m_latestParam, &save);
                 memcpy(&encoder->m_vps.ptl, &saveVPS.ptl, sizeof(saveVPS.ptl));
@@ -406,7 +405,7 @@ int x265_encoder_reconfig_zone(x265_encoder* enc, x265_zone* zone_in)
     return 0;
 }
 
-int x265_encoder_encode(x265_encoder *enc, x265_nal **pp_nal, uint32_t *pi_nal, x265_picture *pic_in, x265_picture *pic_out)
+int x265_encoder_encode(x265_encoder *enc, x265_nal **pp_nal, uint32_t *pi_nal, x265_picture *pic_in, x265_picture **pic_out)
 {
     if (!enc)
         return -1;
@@ -602,7 +601,10 @@ fail:
         *pi_nal = 0;
 
     if (numEncoded && encoder->m_param->csvLogLevel && encoder->m_outputCount >= encoder->m_latestParam->chunkStart)
-        x265_csvlog_frame(encoder->m_param, pic_out);
+    {
+        for (int layer = 0; layer < encoder->m_param->numLayers; layer++)
+            x265_csvlog_frame(encoder->m_param, pic_out[layer]);
+    }
 
     if (numEncoded < 0)
         encoder->m_aborted = true;
@@ -653,11 +655,14 @@ void x265_encoder_log(x265_encoder* enc, int argc, char **argv)
     if (enc)
     {
         Encoder *encoder = static_cast<Encoder*>(enc);
-        x265_stats stats;       
-        encoder->fetchStats(&stats, sizeof(stats));
+        x265_stats stats[MAX_LAYERS];
         int padx = encoder->m_sps.conformanceWindow.rightOffset;
         int pady = encoder->m_sps.conformanceWindow.bottomOffset;
-        x265_csvlog_encode(encoder->m_param, &stats, padx, pady, argc, argv);
+        for (int layer = 0; layer < encoder->m_param->numLayers; layer++)
+        {
+            encoder->fetchStats(stats, sizeof(stats[layer]), layer);
+            x265_csvlog_encode(encoder->m_param, &stats[0], padx, pady, argc, argv);
+        }
     }
 }
 
@@ -744,7 +749,7 @@ int x265_get_slicetype_poc_and_scenecut(x265_encoder *enc, int *slicetype, int *
     if (!enc)
         return -1;
     Encoder *encoder = static_cast<Encoder*>(enc);
-    if (!encoder->copySlicetypePocAndSceneCut(slicetype, poc, sceneCut))
+    if (!encoder->copySlicetypePocAndSceneCut(slicetype, poc, sceneCut, 0))
         return 0;
     return -1;
 }
@@ -1295,7 +1300,7 @@ FILE* x265_csvlog_open(const x265_param* param)
         {
             if (param->csvLogLevel)
             {
-                fprintf(csvfp, "Encode Order, Type, POC, QP, Bits, Scenecut, ");
+                fprintf(csvfp, "Layer , Encode Order, Type, POC, QP, Bits, Scenecut, ");
                 if (!!param->bEnableTemporalSubLayers)
                     fprintf(csvfp, "Temporal Sub Layer ID, ");
                 if (param->csvLogLevel >= 2)
@@ -1409,7 +1414,7 @@ void x265_csvlog_frame(const x265_param* param, const x265_picture* pic)
         return;
 
     const x265_frame_stats* frameStats = &pic->frameData;
-    fprintf(param->csvfpt, "%d, %c-SLICE, %4d, %2.2lf, %10d, %d,", frameStats->encoderOrder, frameStats->sliceType, frameStats->poc,
+    fprintf(param->csvfpt, "%d, %d, %c-SLICE, %4d, %2.2lf, %10d, %d,", pic->layerID, frameStats->encoderOrder, frameStats->sliceType, frameStats->poc,
                                                                    frameStats->qp, (int)frameStats->bits, frameStats->bScenecut);
     if (!!param->bEnableTemporalSubLayers)
         fprintf(param->csvfpt, "%d,", frameStats->tLayer);
@@ -1806,6 +1811,219 @@ fail_or_end:
     return ret;
 }
 
+static enum VmafOutputFormat log_fmt_map(const char *log_fmt)
+{
+	if (log_fmt) {
+		if (!strcmp(log_fmt, "xml"))
+			return VMAF_OUTPUT_FORMAT_XML;
+		if (!strcmp(log_fmt, "json"))
+			return VMAF_OUTPUT_FORMAT_JSON;
+		if (!strcmp(log_fmt, "csv"))
+			return VMAF_OUTPUT_FORMAT_CSV;
+		if (!strcmp(log_fmt, "sub"))
+			return VMAF_OUTPUT_FORMAT_SUB;
+	}
+
+	return VMAF_OUTPUT_FORMAT_NONE;
+}
+
+static enum VmafPoolingMethod pool_method_map(const char *pool_method)
+{
+	if (pool_method) {
+		if (!strcmp(pool_method, "min"))
+			return VMAF_POOL_METHOD_MIN;
+		if (!strcmp(pool_method, "mean"))
+			return VMAF_POOL_METHOD_MEAN;
+		if (!strcmp(pool_method, "harmonic_mean"))
+			return VMAF_POOL_METHOD_HARMONIC_MEAN;
+	}
+	return VMAF_POOL_METHOD_MEAN;
+}
+
+static enum VmafPixelFormat pix_fmt_map(const char *fmt)
+{
+	if (fmt) {
+		if (!strcmp(fmt, "yuv420p") || !strcmp(fmt, "yuv420p10le") || !strcmp(fmt, "yuv420p12le") || !strcmp(fmt, "yuv420p16le"))
+            return VMAF_PIX_FMT_YUV420P;
+        if (!strcmp(fmt, "yuv422p") || !strcmp(fmt, "yuv422p10le"))
+            return VMAF_PIX_FMT_YUV422P;
+        if (!strcmp(fmt, "yuv444p") || !strcmp(fmt, "yuv444p10le"))
+            return VMAF_PIX_FMT_YUV444P;
+	}
+	return VMAF_PIX_FMT_UNKNOWN;
+}
+
+static void copy_picture(float *src, VmafPicture *dst, unsigned width, unsigned height, int src_stride, unsigned bpc)
+{
+    const int bytes_per_value = bpc > 8 ? 2 : 1;
+    const int dst_stride = dst->stride[0] / bytes_per_value;
+    const unsigned b_shift = (bpc > 8) ? (bpc - 8) : 0;
+
+    uint8_t *dst_data = static_cast<uint8_t*>(dst->data[0]);
+
+    for (unsigned i = 0; i < height; i++) {
+        if (bpc > 8) {
+            uint16_t *dst_row = reinterpret_cast<uint16_t*>(dst_data);
+            for (unsigned j = 0; j < width; j++) {
+                dst_row[j] = static_cast<uint16_t>(src[j] * (1 << b_shift));
+            }
+        } else {
+            for (unsigned j = 0; j < width; j++) {
+                dst_data[j] = static_cast<uint8_t>(src[j]);
+            }
+        }
+        src += src_stride / sizeof(float);
+        dst_data += dst_stride * bytes_per_value;
+    }
+}
+
+int load_feature(VmafContext *vmaf, const char *feature_name, VmafFeatureDictionary *d) {
+    int err = vmaf_use_feature(vmaf, feature_name, d);
+    if (err) {
+        printf("problem loading feature extractor: %s\n", feature_name);
+    }
+    return err;
+}
+
+int compute_vmaf(double* vmaf_score, char* fmt, int width, int height, int bitdepth, int(*read_frame)(float *ref_data, float *main_data, float *temp_data, int stride_byte, void *user_data),
+	void *user_data, char *model_path, char *log_path, char *log_fmt, int disable_clip, int disable_avx, int enable_transform, int phone_model, int do_psnr, int do_ssim, int do_ms_ssim,
+	char *pool_method, int n_thread, int n_subsample)
+{
+	int err = 0;
+
+	VmafConfiguration cfg = {
+		.log_level = VMAF_LOG_LEVEL_INFO,
+		.n_threads = n_thread,
+		.n_subsample = n_subsample,
+		.cpumask = disable_avx ? -1 : 0,
+		.gpumask = 0,
+	};
+
+	VmafContext *vmaf;
+	err = vmaf_init(&vmaf, cfg);
+	if (err) {
+		printf("problem initializing VMAF context\n");
+		return -1;
+	}
+
+	uint64_t flags = VMAF_MODEL_FLAGS_DEFAULT;
+	if (disable_clip)
+		flags |= VMAF_MODEL_FLAG_DISABLE_CLIP;
+	if (enable_transform || phone_model)
+		flags |= VMAF_MODEL_FLAG_ENABLE_TRANSFORM;
+
+	VmafModelConfig model_cfg = {
+		.name = "vmaf",
+		.flags = flags,
+	};
+
+	VmafModel *model = NULL;
+	VmafModelCollection *model_collection = NULL;
+
+    int stride = width * sizeof(float);
+    float *ref_data = new float[height * stride];
+    float *main_data = new float[height * stride];
+    float *temp_data = new float[height * stride];
+    enum VmafOutputFormat output_fmt = log_fmt_map(log_fmt);
+
+	err = vmaf_model_load_from_path(&model, &model_cfg, model_path);
+	if (err) {
+		printf("problem loading model file: %s\n", model_path);
+		goto end;
+	}
+	err = vmaf_use_features_from_model(vmaf, model);
+	if (err) {
+		printf("problem loading feature extractors from model file: %s\n", model_path);
+		goto end;
+	}
+
+	if (do_psnr) {
+		VmafFeatureDictionary *d = NULL;
+		vmaf_feature_dictionary_set(&d, "enable_chroma", "false");
+		err = load_feature(vmaf, "psnr", d);
+		if (err) goto end;
+	}
+
+	if (do_ssim) {
+		err = load_feature(vmaf, "float_ssim", NULL);
+		if (err) goto end;
+	}
+
+	if (do_ms_ssim) {
+		err = load_feature(vmaf, "float_ms_ssim", NULL);
+		if (err) goto end;
+	}
+
+	if (!ref_data || !main_data || !temp_data) {
+		printf("problem allocating picture memory\n");
+		err = -1;
+		goto free_data;
+	}
+	unsigned picture_index;
+	for (picture_index = 0;; picture_index++) {
+		err = read_frame(ref_data, main_data, temp_data, stride, user_data);
+		if (err == 1) {
+			printf("problem during read_frame\n");
+			goto free_data;
+		}
+		else if (err == 2) break;
+
+		VmafPicture pic_ref, pic_dist;
+		err = vmaf_picture_alloc(&pic_ref, pix_fmt_map(fmt), bitdepth, width, height);
+		err |= vmaf_picture_alloc(&pic_dist, pix_fmt_map(fmt), bitdepth, width, height);
+		if (err) {
+			printf("problem allocating picture memory\n");
+			vmaf_picture_unref(&pic_ref);
+			vmaf_picture_unref(&pic_dist);
+			goto free_data;
+		}
+
+		const unsigned bpc = bitdepth;
+			copy_picture(ref_data, &pic_ref, width, height, stride, bpc);
+			copy_picture(main_data, &pic_dist, width, height, stride, bpc);
+
+		err = vmaf_read_pictures(vmaf, &pic_ref, &pic_dist, picture_index);
+		if (err) {
+			printf("problem reading pictures\n");
+			break;
+		}
+	}
+
+	err = vmaf_read_pictures(vmaf, NULL, NULL, 0);
+	if (err) {
+		printf("problem flushing context\n");
+		return err;
+	}
+
+	err = vmaf_score_pooled(vmaf, model, pool_method_map(pool_method), vmaf_score, 0, picture_index - 1);
+	if (err) {
+		printf("problem generating pooled VMAF score\n");
+		goto free_data;
+	}
+
+	if (output_fmt == VMAF_OUTPUT_FORMAT_NONE && log_path) {
+		output_fmt = VMAF_OUTPUT_FORMAT_XML;
+		printf("use default log_fmt xml");
+	}
+	if (output_fmt) {
+		err = vmaf_write_output(vmaf, log_path, output_fmt);
+		if (err) {
+			printf("could not write output: %s\n", log_path);
+			goto free_data;
+		}
+	}
+
+free_data:
+    delete[] ref_data;
+    delete[] main_data;
+    delete[] temp_data;
+end:
+    vmaf_model_destroy(model);
+    vmaf_model_collection_destroy(model_collection);
+    vmaf_close(vmaf);
+    return err;
+}
+
 int read_frame(float *reference_data, float *distorted_data, float *temp_data, int stride_byte, void *s)
 {
     x265_vmaf_data *user_data = (x265_vmaf_data *)s;
@@ -1921,15 +2139,22 @@ double x265_calculate_vmafscore(x265_param *param, x265_vmaf_data *data)
         if ((param->sourceWidth * param->sourceHeight) % 2 != 0)
             x265_log(NULL, X265_LOG_ERROR, "Invalid file size\n");
         data->offset = param->sourceWidth * param->sourceHeight / 2;
+        vcd->format = "yuv420p";
     }
     else if (param->internalCsp == X265_CSP_I422)
+    {
         data->offset = param->sourceWidth * param->sourceHeight;
+        vcd->format = "yuv422p10le";
+    }
     else if (param->internalCsp == X265_CSP_I444)
+    {
         data->offset = param->sourceWidth * param->sourceHeight * 2;
+		vcd->format = "yuv444p10le";
+    }
     else
         x265_log(NULL, X265_LOG_ERROR, "Invalid format\n");
 
-    compute_vmaf(&score, vcd->format, data->width, data->height, read_frame, data, vcd->model_path, vcd->log_path, vcd->log_fmt, vcd->disable_clip, vcd->disable_avx, vcd->enable_transform, vcd->phone_model, vcd->psnr, vcd->ssim, vcd->ms_ssim, vcd->pool, vcd->thread, vcd->subsample, vcd->enable_conf_interval);
+    compute_vmaf(&score, vcd->format, data->width, data->height, param->sourceBitDepth, read_frame, data, vcd->model_path, vcd->log_path, vcd->log_fmt, vcd->disable_clip, vcd->disable_avx, vcd->enable_transform, vcd->phone_model, vcd->psnr, vcd->ssim, vcd->ms_ssim, vcd->pool, vcd->thread, vcd->subsample);
 
     return score;
 }
@@ -2022,16 +2247,24 @@ int read_frame_8bit(float *reference_data, float *distorted_data, float *temp_da
     return 2;
 }
 
-double x265_calculate_vmaf_framelevelscore(x265_vmaf_framedata *vmafframedata)
+double x265_calculate_vmaf_framelevelscore(x265_param *param, x265_vmaf_framedata *vmafframedata)
 {
-    double score; 
+    double score;
+
+    if (param->internalCsp == X265_CSP_I420)
+        vcd->format = "yuv420p";
+    else if (param->internalCsp == X265_CSP_I422)
+        vcd->format = "yuv422p10le";
+    else
+		vcd->format = "yuv444p10le";
+
     int (*read_frame)(float *reference_data, float *distorted_data, float *temp_data,
                       int stride, void *s);
     if (vmafframedata->internalBitDepth == 8)
         read_frame = read_frame_8bit;
     else
         read_frame = read_frame_10bit;
-    compute_vmaf(&score, vcd->format, vmafframedata->width, vmafframedata->height, read_frame, vmafframedata, vcd->model_path, vcd->log_path, vcd->log_fmt, vcd->disable_clip, vcd->disable_avx, vcd->enable_transform, vcd->phone_model, vcd->psnr, vcd->ssim, vcd->ms_ssim, vcd->pool, vcd->thread, vcd->subsample, vcd->enable_conf_interval);
+    compute_vmaf(&score, vcd->format, vmafframedata->width, vmafframedata->height, param->sourceBitDepth, read_frame, vmafframedata, vcd->model_path, vcd->log_path, vcd->log_fmt, vcd->disable_clip, vcd->disable_avx, vcd->enable_transform, vcd->phone_model, vcd->psnr, vcd->ssim, vcd->ms_ssim, vcd->pool, vcd->thread, vcd->subsample);
 
     return score;
 }
