@@ -53,8 +53,8 @@ DPB::~DPB()
         FrameData* next = m_frameDataFreeList->m_freeListNext;
         m_frameDataFreeList->destroy();
 
-        m_frameDataFreeList->m_reconPic->destroy();
-        delete m_frameDataFreeList->m_reconPic;
+        m_frameDataFreeList->m_reconPic[0]->destroy();
+        delete m_frameDataFreeList->m_reconPic[0];
 
         delete m_frameDataFreeList;
         m_frameDataFreeList = next;
@@ -75,7 +75,7 @@ void DPB::recycleUnreferenced()
         if (curFrame->m_param->bEnableTemporalFilter)
             isMCSTFReferenced =!!(curFrame->m_refPicCnt[1]);
 
-        if (!curFrame->m_encData->m_bHasReferences && !curFrame->m_countRefEncoders && !isMCSTFReferenced)
+        if (curFrame->m_valid && !curFrame->m_encData->m_bHasReferences && !curFrame->m_countRefEncoders && !isMCSTFReferenced)
         {
             curFrame->m_bChromaExtended = false;
 
@@ -95,6 +95,12 @@ void DPB::recycleUnreferenced()
 
             // iterator is invalidated by remove, restart scan
             m_picList.remove(*curFrame);
+#if ENABLE_MULTIVIEW
+            if (curFrame->m_param->numViews > 1 && !curFrame->m_viewId && m_picList.getPOC(curFrame->m_poc, 1) && curFrame == m_picList.getPOC(curFrame->m_poc, 1)->refPicSetInterLayer0.getPOC(curFrame->m_poc, curFrame->m_viewId))
+            {
+                m_picList.getPOC(curFrame->m_poc, 1)->refPicSetInterLayer0.removeSubDPB(*curFrame);
+            }
+#endif
             iterFrame = m_picList.first();
 
             m_freeList.pushBack(*curFrame);
@@ -126,7 +132,8 @@ void DPB::recycleUnreferenced()
                 curFrame->m_prevCtuInfoChange = NULL;
             }
             curFrame->m_encData = NULL;
-            curFrame->m_reconPic = NULL;
+            for (int i = 0; i < !!curFrame->m_param->bEnableSCC + 1; i++)
+                curFrame->m_reconPic[i] = NULL;
         }
     }
 }
@@ -145,6 +152,11 @@ void DPB::prepareEncode(Frame *newFrame)
         m_lastIDR = pocCurr;
     slice->m_lastIDR = m_lastIDR;
     slice->m_sliceType = IS_X265_TYPE_B(type) ? B_SLICE : (type == X265_TYPE_P) ? P_SLICE : I_SLICE;
+#if ENABLE_SCC_EXT
+    if (slice->m_param->bEnableSCC)        slice->m_origSliceType = slice->m_sliceType;
+    if (slice->m_param->bEnableSCC && IS_X265_TYPE_I(type))
+        slice->m_sliceType = P_SLICE;
+#endif
 
     if (type == X265_TYPE_B)
     {
@@ -177,7 +189,8 @@ void DPB::prepareEncode(Frame *newFrame)
 
     m_picList.pushFront(*newFrame);
 
-    if (m_bTemporalSublayer && getTemporalLayerNonReferenceFlag())
+    int layer = slice->m_param->numViews > 1 ? newFrame->m_viewId : (slice->m_param->numScalableLayers > 1) ? newFrame->m_sLayerId : 0;
+    if (m_bTemporalSublayer && getTemporalLayerNonReferenceFlag(layer))
     {
         switch (slice->m_nalUnitType)
         {
@@ -195,12 +208,13 @@ void DPB::prepareEncode(Frame *newFrame)
         }
     }
     // Do decoding refresh marking if any
-    decodingRefreshMarking(pocCurr, slice->m_nalUnitType);
+    decodingRefreshMarking(pocCurr, slice->m_nalUnitType, layer);
 
-    computeRPS(pocCurr, newFrame->m_tempLayer, slice->isIRAP(), &slice->m_rps, slice->m_sps->maxDecPicBuffering[newFrame->m_tempLayer]);
+    uint32_t maxDecBuffer = (slice->m_sps->maxDecPicBuffering[newFrame->m_tempLayer] >= 8 && slice->m_param->bEnableSCC) ? 7 : slice->m_sps->maxDecPicBuffering[newFrame->m_tempLayer];
+    computeRPS(pocCurr, newFrame->m_tempLayer, slice->isIRAP(), &slice->m_rps, maxDecBuffer, layer);
     bool isTSAPic = ((slice->m_nalUnitType == 2) || (slice->m_nalUnitType == 3)) ? true : false;
     // Mark pictures in m_piclist as unreferenced if they are not included in RPS
-    applyReferencePictureSet(&slice->m_rps, pocCurr, newFrame->m_tempLayer, isTSAPic);
+    applyReferencePictureSet(&slice->m_rps, pocCurr, newFrame->m_tempLayer, isTSAPic, layer);
 
 
     if (m_bTemporalSublayer && newFrame->m_tempLayer > 0
@@ -210,9 +224,9 @@ void DPB::prepareEncode(Frame *newFrame)
             || slice->m_nalUnitType == NAL_UNIT_CODED_SLICE_RASL_R)
         )
     {
-        if (isTemporalLayerSwitchingPoint(pocCurr, newFrame->m_tempLayer) || (slice->m_sps->maxTempSubLayers == 1))
+        if (isTemporalLayerSwitchingPoint(pocCurr, newFrame->m_tempLayer, layer) || (slice->m_sps->maxTempSubLayers == 1))
         {
-            if (getTemporalLayerNonReferenceFlag())
+            if (getTemporalLayerNonReferenceFlag(layer))
             {
                 slice->m_nalUnitType = NAL_UNIT_CODED_SLICE_TSA_N;
             }
@@ -221,7 +235,7 @@ void DPB::prepareEncode(Frame *newFrame)
                 slice->m_nalUnitType = NAL_UNIT_CODED_SLICE_TSA_R;
             }
         }
-        else if (isStepwiseTemporalLayerSwitchingPoint(&slice->m_rps, pocCurr, newFrame->m_tempLayer))
+        else if (isStepwiseTemporalLayerSwitchingPoint(&slice->m_rps, pocCurr, newFrame->m_tempLayer, layer))
         {
             bool isSTSA = true;
             int id = newFrame->m_gopOffset % x265_gop_ra_length[newFrame->m_gopId];
@@ -254,7 +268,7 @@ void DPB::prepareEncode(Frame *newFrame)
             }
             if (isSTSA == true)
             {
-                if (getTemporalLayerNonReferenceFlag())
+                if (getTemporalLayerNonReferenceFlag(layer))
                 {
                     slice->m_nalUnitType = NAL_UNIT_CODED_SLICE_STSA_N;
                 }
@@ -266,12 +280,22 @@ void DPB::prepareEncode(Frame *newFrame)
         }
     }
 
+#if ENABLE_MULTIVIEW
+    if (newFrame->m_viewId)
+        slice->createInterLayerReferencePictureSet(m_picList, newFrame->refPicSetInterLayer0, newFrame->refPicSetInterLayer1);
+#endif
+    int numRef = slice->m_param->bEnableSCC ? slice->m_rps.numberOfNegativePictures + 1 : slice->m_rps.numberOfNegativePictures;
     if (slice->m_sliceType != I_SLICE)
-        slice->m_numRefIdx[0] = x265_clip3(1, newFrame->m_param->maxNumReferences, slice->m_rps.numberOfNegativePictures);
+        slice->m_numRefIdx[0] = x265_clip3(1, newFrame->m_param->maxNumReferences, numRef + newFrame->refPicSetInterLayer0.size() + newFrame->refPicSetInterLayer1.size());
     else
-        slice->m_numRefIdx[0] = X265_MIN(newFrame->m_param->maxNumReferences, slice->m_rps.numberOfNegativePictures); // Ensuring L0 contains just the -ve POC
-    slice->m_numRefIdx[1] = X265_MIN(newFrame->m_param->bBPyramid ? 2 : 1, slice->m_rps.numberOfPositivePictures);
-    slice->setRefPicList(m_picList);
+        slice->m_numRefIdx[0] = X265_MIN(newFrame->m_param->maxNumReferences, numRef); // Ensuring L0 contains just the -ve POC
+#if ENABLE_MULTIVIEW || ENABLE_SCC_EXT
+    if(slice->m_param->numViews > 1 || !!slice->m_param->bEnableSCC)
+        slice->m_numRefIdx[1] = X265_MIN(newFrame->m_param->bBPyramid ? 3 : 2, slice->m_rps.numberOfPositivePictures + newFrame->refPicSetInterLayer0.size() + newFrame->refPicSetInterLayer1.size());
+    else
+#endif
+        slice->m_numRefIdx[1] = X265_MIN(newFrame->m_param->bBPyramid ? 2 : 1, slice->m_rps.numberOfPositivePictures);
+    slice->setRefPicList(m_picList, newFrame->refPicSetInterLayer0, newFrame->refPicSetInterLayer1, layer);
 
     X265_CHECK(slice->m_sliceType != B_SLICE || slice->m_numRefIdx[1], "B slice without L1 references (non-fatal)\n");
 
@@ -280,9 +304,29 @@ void DPB::prepareEncode(Frame *newFrame)
         /* TODO: the lookahead should be able to tell which reference picture
          * had the least motion residual.  We should be able to use that here to
          * select a colocation reference list and index */
-        slice->m_colFromL0Flag = false;
+
+        bool bLowDelay = true;
+        int  iCurrPOC = slice->m_poc;
+        int iRefIdx = 0;
+
+        for (iRefIdx = 0; iRefIdx < slice->m_numRefIdx[0] && bLowDelay; iRefIdx++)
+        {
+            if (slice->m_refPOCList[0][iRefIdx] > iCurrPOC)
+            {
+                bLowDelay = false;
+            }
+        }
+        for (iRefIdx = 0; iRefIdx < slice->m_numRefIdx[1] && bLowDelay; iRefIdx++)
+        {
+            if (slice->m_refPOCList[1][iRefIdx] > iCurrPOC)
+            {
+                bLowDelay = false;
+            }
+        }
+
+        slice->m_bCheckLDC = bLowDelay;
+        slice->m_colFromL0Flag = bLowDelay;
         slice->m_colRefIdx = 0;
-        slice->m_bCheckLDC = false;
     }
     else
     {
@@ -291,6 +335,59 @@ void DPB::prepareEncode(Frame *newFrame)
         slice->m_colRefIdx = 0;
     }
 
+    slice->m_bTemporalMvp = slice->m_sps->bTemporalMVPEnabled;
+#if ENABLE_SCC_EXT
+    bool bGPBcheck = false;
+    if (slice->m_sliceType == B_SLICE)
+    {
+        if (slice->m_param->bEnableSCC)
+        {
+            if (slice->m_numRefIdx[0] - 1 == slice->m_numRefIdx[1])
+            {
+                bGPBcheck = true;
+                for (int i = 0; i < slice->m_numRefIdx[1]; i++)
+                {
+                    if (slice->m_refPOCList[1][i] != slice->m_refPOCList[0][i])
+                    {
+                        bGPBcheck = false;
+                        break;
+                    }
+                }
+            }
+        }
+        else if (slice->m_numRefIdx[0] == slice->m_numRefIdx[1])
+        {
+            bGPBcheck = true;
+            int i;
+            for (i = 0; i < slice->m_numRefIdx[1]; i++)
+            {
+                if (slice->m_refPOCList[1][i] != slice->m_refPOCList[0][i])
+                {
+                    bGPBcheck = false;
+                    break;
+                }
+            }
+        }
+    }
+    if (bGPBcheck)
+    {
+        slice->m_bLMvdL1Zero = true;
+    }
+    else
+    {
+        slice->m_bLMvdL1Zero = false;
+    }
+
+    if (!slice->isIntra() && slice->m_param->bEnableTemporalMvp)
+    {
+        const Frame* colPic = slice->m_refFrameList[slice->isInterB() && !slice->m_colFromL0Flag][slice->m_colRefIdx];
+        if (colPic->m_poc == slice->m_poc)
+            slice->m_bTemporalMvp = false;
+        else
+            slice->m_bTemporalMvp = true;
+    }
+#endif
+
     // Disable Loopfilter in bound area, because we will do slice-parallelism in future
     slice->m_sLFaseFlag = (newFrame->m_param->maxSlices > 1) ? false : ((SLFASE_CONSTANT & (1 << (pocCurr % 31))) > 0);
 
@@ -308,7 +405,7 @@ void DPB::prepareEncode(Frame *newFrame)
     }
 }
 
-void DPB::computeRPS(int curPoc, int tempId, bool isRAP, RPS * rps, unsigned int maxDecPicBuffer)
+void DPB::computeRPS(int curPoc, int tempId, bool isRAP, RPS * rps, unsigned int maxDecPicBuffer, int scalableLayerId)
 {
     unsigned int poci = 0, numNeg = 0, numPos = 0;
 
@@ -316,10 +413,18 @@ void DPB::computeRPS(int curPoc, int tempId, bool isRAP, RPS * rps, unsigned int
 
     while (iterPic && (poci < maxDecPicBuffer - 1))
     {
-        if ((iterPic->m_poc != curPoc) && iterPic->m_encData->m_bHasReferences)
+        int layer = iterPic->m_param->numViews > 1 ? iterPic->m_viewId : (iterPic->m_param->numScalableLayers > 1) ? iterPic->m_sLayerId : 0;
+        if (iterPic->m_valid && (iterPic->m_poc != curPoc) && iterPic->m_encData->m_bHasReferences && layer == scalableLayerId)
         {
             if ((!m_bTemporalSublayer || (iterPic->m_tempLayer <= tempId)) && ((m_lastIDR >= curPoc) || (m_lastIDR <= iterPic->m_poc)))
             {
+#if ENABLE_MULTIVIEW
+                    if (iterPic->m_param->numViews > 1 && layer && numNeg == iterPic->m_param->maxNumReferences - 1 && (iterPic->m_poc - curPoc) < 0)
+                    {
+                        iterPic = iterPic->m_next;
+                        continue;
+                    }
+#endif
                     rps->poc[poci] = iterPic->m_poc;
                     rps->deltaPOC[poci] = rps->poc[poci] - curPoc;
                     (rps->deltaPOC[poci] < 0) ? numNeg++ : numPos++;
@@ -337,10 +442,11 @@ void DPB::computeRPS(int curPoc, int tempId, bool isRAP, RPS * rps, unsigned int
     rps->sortDeltaPOC();
 }
 
-bool DPB::getTemporalLayerNonReferenceFlag()
+bool DPB::getTemporalLayerNonReferenceFlag(int scalableLayerId)
 {
     Frame* curFrame = m_picList.first();
-    if (curFrame->m_encData->m_bHasReferences)
+    int layer = curFrame->m_param->numViews > 1 ? curFrame->m_viewId : (curFrame->m_param->numScalableLayers > 1) ? curFrame->m_sLayerId : 0;
+    if (curFrame->m_valid && curFrame->m_encData->m_bHasReferences && layer == scalableLayerId)
     {
         curFrame->m_sameLayerRefPic = true;
         return false;
@@ -350,7 +456,7 @@ bool DPB::getTemporalLayerNonReferenceFlag()
 }
 
 /* Marking reference pictures when an IDR/CRA is encountered. */
-void DPB::decodingRefreshMarking(int pocCurr, NalUnitType nalUnitType)
+void DPB::decodingRefreshMarking(int pocCurr, NalUnitType nalUnitType, int scalableLayerId)
 {
     if (nalUnitType == NAL_UNIT_CODED_SLICE_IDR_W_RADL || nalUnitType == NAL_UNIT_CODED_SLICE_IDR_N_LP)
     {
@@ -359,7 +465,8 @@ void DPB::decodingRefreshMarking(int pocCurr, NalUnitType nalUnitType)
         Frame* iterFrame = m_picList.first();
         while (iterFrame)
         {
-            if (iterFrame->m_poc != pocCurr)
+            int layer = iterFrame->m_param->numViews > 1 ? iterFrame->m_viewId : (iterFrame->m_param->numScalableLayers > 1) ? iterFrame->m_sLayerId : 0;
+            if (iterFrame->m_valid && iterFrame->m_poc != pocCurr && layer == scalableLayerId)
                 iterFrame->m_encData->m_bHasReferences = false;
             iterFrame = iterFrame->m_next;
         }
@@ -376,12 +483,14 @@ void DPB::decodingRefreshMarking(int pocCurr, NalUnitType nalUnitType)
             Frame* iterFrame = m_picList.first();
             while (iterFrame)
             {
-                if (iterFrame->m_poc != pocCurr && iterFrame->m_poc != m_pocCRA)
+                int layer = iterFrame->m_param->numViews > 1 ? iterFrame->m_viewId : (iterFrame->m_param->numScalableLayers > 1) ? iterFrame->m_sLayerId : 0;
+                if (iterFrame->m_valid && iterFrame->m_poc != pocCurr && iterFrame->m_poc != m_pocCRA && layer == scalableLayerId)
                     iterFrame->m_encData->m_bHasReferences = false;
                 iterFrame = iterFrame->m_next;
             }
 
-            m_bRefreshPending = false;
+            if (scalableLayerId == m_picList.first()->m_param->numLayers - 1)
+                m_bRefreshPending = false;
         }
         if (nalUnitType == NAL_UNIT_CODED_SLICE_CRA)
         {
@@ -398,13 +507,14 @@ void DPB::decodingRefreshMarking(int pocCurr, NalUnitType nalUnitType)
 }
 
 /** Function for applying picture marking based on the Reference Picture Set */
-void DPB::applyReferencePictureSet(RPS *rps, int curPoc, int tempId, bool isTSAPicture)
+void DPB::applyReferencePictureSet(RPS *rps, int curPoc, int tempId, bool isTSAPicture, int scalableLayerId)
 {
     // loop through all pictures in the reference picture buffer
     Frame* iterFrame = m_picList.first();
     while (iterFrame)
     {
-        if (iterFrame->m_poc != curPoc && iterFrame->m_encData->m_bHasReferences)
+        int layer = iterFrame->m_param->numViews > 1 ? iterFrame->m_viewId : (iterFrame->m_param->numScalableLayers > 1) ? iterFrame->m_sLayerId : 0;
+        if (iterFrame->m_valid && iterFrame->m_poc != curPoc && iterFrame->m_encData->m_bHasReferences && layer == scalableLayerId)
         {
             // loop through all pictures in the Reference Picture Set
             // to see if the picture should be kept as reference picture
@@ -441,13 +551,14 @@ void DPB::applyReferencePictureSet(RPS *rps, int curPoc, int tempId, bool isTSAP
     }
 }
 
-bool DPB::isTemporalLayerSwitchingPoint(int curPoc, int tempId)
+bool DPB::isTemporalLayerSwitchingPoint(int curPoc, int tempId, int scalableLayerId)
 {
     // loop through all pictures in the reference picture buffer
     Frame* iterFrame = m_picList.first();
     while (iterFrame)
     {
-        if (iterFrame->m_poc != curPoc && iterFrame->m_encData->m_bHasReferences)
+        int layer = iterFrame->m_param->numViews > 1 ? iterFrame->m_viewId : (iterFrame->m_param->numScalableLayers > 1) ? iterFrame->m_sLayerId : 0;
+        if (iterFrame->m_valid && iterFrame->m_poc != curPoc && iterFrame->m_encData->m_bHasReferences && layer == scalableLayerId)
         {
             if (iterFrame->m_tempLayer >= tempId)
             {
@@ -459,13 +570,14 @@ bool DPB::isTemporalLayerSwitchingPoint(int curPoc, int tempId)
     return true;
 }
 
-bool DPB::isStepwiseTemporalLayerSwitchingPoint(RPS *rps, int curPoc, int tempId)
+bool DPB::isStepwiseTemporalLayerSwitchingPoint(RPS *rps, int curPoc, int tempId, int scalableLayerId)
 {
     // loop through all pictures in the reference picture buffer
     Frame* iterFrame = m_picList.first();
     while (iterFrame)
     {
-        if (iterFrame->m_poc != curPoc && iterFrame->m_encData->m_bHasReferences)
+        int layer = iterFrame->m_param->numViews > 1 ? iterFrame->m_viewId : (iterFrame->m_param->numScalableLayers > 1) ? iterFrame->m_sLayerId : 0;
+        if (iterFrame->m_valid && iterFrame->m_poc != curPoc && iterFrame->m_encData->m_bHasReferences && layer == scalableLayerId)
         {
             for (int i = 0; i < rps->numberOfPositivePictures + rps->numberOfNegativePictures; i++)
             {
