@@ -156,12 +156,12 @@ public:
     void destroy();
 
     /* triggers encode of a new frame by the worker thread */
-    bool startCompressFrame(Frame* curFrame);
+    bool startCompressFrame(Frame* curFrame[MAX_LAYERS]);
 
     /* blocks until worker thread is done, returns access unit */
-    Frame *getEncodedPicture(NALList& list);
+    Frame **getEncodedPicture(NALList& list);
 
-    void initDecodedPictureHashSEI(int row, int cuAddr, int height);
+    void initDecodedPictureHashSEI(int row, int cuAddr, int height, int layer);
 
     Event                    m_enable;
     Event                    m_done;
@@ -190,34 +190,35 @@ public:
     RateControlEntry         m_rce;
     SEIDecodedPictureHash    m_seiReconPictureDigest;
 
-    uint64_t                 m_SSDY;
-    uint64_t                 m_SSDU;
-    uint64_t                 m_SSDV;
-    double                   m_ssim;
-    uint64_t                 m_accessUnitBits;
-    uint32_t                 m_ssimCnt;
+    uint64_t                 m_SSDY[MAX_LAYERS];
+    uint64_t                 m_SSDU[MAX_LAYERS];
+    uint64_t                 m_SSDV[MAX_LAYERS];
+    double                   m_ssim[MAX_LAYERS];
+    uint64_t                 m_accessUnitBits[MAX_LAYERS];
+    uint32_t                 m_ssimCnt[MAX_LAYERS];
 
     volatile int             m_activeWorkerCount;        // count of workers currently encoding or filtering CTUs
     volatile int             m_totalActiveWorkerCount;   // sum of m_activeWorkerCount sampled at end of each CTU
     volatile int             m_activeWorkerCountSamples; // count of times m_activeWorkerCount was sampled (think vbv restarts)
     volatile int             m_countRowBlocks;           // count of workers forced to abandon a row because of top dependency
-    int64_t                  m_startCompressTime;        // timestamp when frame encoder is given a frame
-    int64_t                  m_row0WaitTime;             // timestamp when row 0 is allowed to start
-    int64_t                  m_allRowsAvailableTime;     // timestamp when all reference dependencies are resolved
-    int64_t                  m_endCompressTime;          // timestamp after all CTUs are compressed
-    int64_t                  m_endFrameTime;             // timestamp after RCEnd, NR updates, etc
-    int64_t                  m_stallStartTime;           // timestamp when worker count becomes 0
-    int64_t                  m_prevOutputTime;           // timestamp when prev frame was retrieved by API thread
-    int64_t                  m_slicetypeWaitTime;        // total elapsed time waiting for decided frame
-    int64_t                  m_totalWorkerElapsedTime;   // total elapsed time spent by worker threads processing CTUs
-    int64_t                  m_totalNoWorkerTime;        // total elapsed time without any active worker threads
+    int64_t                  m_startCompressTime[MAX_LAYERS];        // timestamp when frame encoder is given a frame
+    int64_t                  m_row0WaitTime[MAX_LAYERS];             // timestamp when row 0 is allowed to start
+    int64_t                  m_allRowsAvailableTime[MAX_LAYERS];     // timestamp when all reference dependencies are resolved
+    int64_t                  m_endCompressTime[MAX_LAYERS];          // timestamp after all CTUs are compressed
+    int64_t                  m_endFrameTime[MAX_LAYERS];             // timestamp after RCEnd, NR updates, etc
+    int64_t                  m_stallStartTime[MAX_LAYERS];           // timestamp when worker count becomes 0
+    int64_t                  m_prevOutputTime[MAX_LAYERS];           // timestamp when prev frame was retrieved by API thread
+    int64_t                  m_slicetypeWaitTime[MAX_LAYERS];        // total elapsed time waiting for decided frame
+    int64_t                  m_totalWorkerElapsedTime[MAX_LAYERS];   // total elapsed time spent by worker threads processing CTUs
+    int64_t                  m_totalNoWorkerTime[MAX_LAYERS];        // total elapsed time without any active worker threads
 #if DETAILED_CU_STATS
     CUStats                  m_cuStats;
 #endif
 
     Encoder*                 m_top;
     x265_param*              m_param;
-    Frame*                   m_frame;
+    Frame*                   m_frame[MAX_LAYERS];
+    Frame**                  m_retFrameBuffer;
     NoiseReduction*          m_nr;
     ThreadLocalData*         m_tld; /* for --no-wpp */
     Bitstream*               m_outStreams;
@@ -238,6 +239,8 @@ public:
     TemporalFilter*          m_frameEncTF;
     TemporalFilterRefPicInfo m_mcstfRefList[MAX_MCSTF_TEMPORAL_WINDOW_LENGTH];
 
+    int                      m_sLayerId;
+
     class WeightAnalysis : public BondedTaskGroup
     {
     public:
@@ -258,20 +261,20 @@ protected:
     bool initializeGeoms();
 
     /* analyze / compress frame, can be run in parallel within reference constraints */
-    void compressFrame();
+    void compressFrame(int layer);
 
     /* called by compressFrame to generate final per-row bitstreams */
-    void encodeSlice(uint32_t sliceAddr);
+    void encodeSlice(uint32_t sliceAddr, int layer);
 
     void threadMain();
     int  collectCTUStatistics(const CUData& ctu, FrameStats* frameLog);
     void noiseReductionUpdate();
-    void writeTrailingSEIMessages();
+    void writeTrailingSEIMessages(int layer);
     bool writeToneMapInfo(x265_sei_payload *payload);
 
     /* Called by WaveFront::findJob() */
-    virtual void processRow(int row, int threadId);
-    virtual void processRowEncoder(int row, ThreadLocalData& tld);
+    virtual void processRow(int row, int threadId, int layer);
+    virtual void processRowEncoder(int row, ThreadLocalData& tld, int layer);
 
     void enqueueRowEncoder(int row) { WaveFront::enqueueRow(row * 2 + 0); }
     void enqueueRowFilter(int row)  { WaveFront::enqueueRow(row * 2 + 1); }
@@ -280,8 +283,8 @@ protected:
 #if ENABLE_LIBVMAF
     void vmafFrameLevelScore();
 #endif
-    void collectDynDataFrame();
-    void computeAvgTrainingData();
+    void collectDynDataFrame(int layer);
+    void computeAvgTrainingData(int layer);
     void collectDynDataRow(CUData& ctu, FrameStats* rowStats);    
     void readModel(FilmGrainCharacteristics* m_filmGrain, FILE* filmgrain);
 };
