@@ -128,8 +128,8 @@ public:
 
     void start(Frame *pic, Entropy& initState);
 
-    void processRow(int row);
-    void processPostRow(int row);
+    void processRow(int row, int layer);
+    void processPostRow(int row, int layer);
     void computeMEIntegral(int row);
 };
 }
